<?php
// $Id: translatable.database.inc,v 1.1 2008/02/17 05:22:40 sun Exp $

/**
 * Returns database definitions for a given object type.
 * 
 * @param string $object
 *   An object name, currently either 'node' or 'translation'.
 * 
 * @return array
 *   An array containing table name and key of the object, optionally including
 *   a parent key and additional table columns.
 */
function translatable_get_object_definition(&$object) {
  // If $object is an array, it contains the definition already.
  if (is_array($object)) {
    return;
  }
  switch ($object) {
    case 'node':
      $object = array(
        'table' => 'node_translatable',
        'key' => 'nid',
        'parentkey' => 'tnid',
        'language' => 'language',
        'any' => 'any',
      );
      return;
    
    case 'translation':
      $object = array(
        'table' => 'translatable_object',
        'key' => 'tid',
        'object_name' => 'object_name',
        'object_key' => 'object_key',
        'object_field' => 'object_field',
        'translation' => 'translation',
        'locale' => 'locale',
      );
      return;
  }
}

/**
 * Find a content translation in translation index.
 * 
 * @param string $object
 *   The object name to search for.
 * @param string $conditions
 *   A SQL where clause to search for.
 * @param bool $all
 *   Whether to return one or all query result rows.
 * @param bool $force
 *   Whether to omit cached query results.
 * @param array $args
 *   Additional query arguments, supplied as keys 'fields', 'group by', and
 *   'having'.
 * 
 * @return array
 *   An array containing the query result(s).
 */
function translatable_find($object, $conditions = '', $all = TRUE, $force = FALSE, $args = array()) {
  static $localcache = array();
  
  $cid = $conditions .':'. ($all ? 'all' : 'one');
  if (isset($localcache[$cid]) && !$force) {
    return $localcache[$cid];
  }
  
  translatable_get_object_definition($object);
  
  $fields = '*'. (isset($args['fields']) ? ', '. implode(', ', (array)$args['fields']) : '');
  $sql = 'SELECT '. $fields .' FROM {'. $object['table'] .'}';
  if ($conditions) {
    $sql .= ' WHERE '. $conditions;
  }
  if (isset($args['group by'])) {
    $sql .= ' GROUP BY '. implode(', ', (array)$args['group by']);
  }
  if (isset($args['having'])) {
    $sql .= ' HAVING '. implode(' AND ', (array)$args['having']);
  }
  if (!$all) {
    $result = db_query_range($sql, 0, 1);
  }
  else {
    $result = db_query($sql);
  }
  
  // Parse database results into object specific result sets.
  unset($object['table']);
  $key = $object['key'];
  $items = array();
  while ($item = db_fetch_array($result)) {
    foreach ($object as $column) {
      $items[$item[$key]][$column] = $item[$column];
    }
  }
  
  if (!$all) {
    $oneitem = array();
    foreach ($items as $object['key'] => $item) {
      foreach ($item as $key => $value) {
        $oneitem[$key] = $value;
      }
      break;
    }
    $localcache[$cid] = $oneitem;
    return $oneitem;
  }
  else {
    $localcache[$cid] = $items;
    return $items;
  }
}

/**
 * Fetch a single content from translation index.
 */
function translatable_findbyid($object, $id, $force = FALSE) {
  translatable_get_object_definition($object);
  return translatable_find($object, $object['key'] .' = '. (int)$id, FALSE, $force);
}

/**
 * Save a content to translation index.
 */
function translatable_save($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  translatable_get_object_definition($object);

  if ($object['table'] == 'translatable_object' && !$item[$object['key']]) {
    $translation = translatable_find('translation', "object_name='". $item['object_name'] ."' AND object_key='". $item['object_key'] ."' AND object_field='". $item['object_field'] ."' AND locale='". $item['locale'] ."'", FALSE);
    $item[$object['key']] = $translation[$object['key']];
  }
  
  if (db_result(db_query("SELECT COUNT(%s) FROM {%s} WHERE %s = %d", $object['key'], $object['table'], $object['key'], $item[$object['key']])) > 0) {
    return translatable_update($object, $item);
  }
  else {
    if (isset($object['parentkey']) && !$item[$object['parentkey']]) {
      // If no source content id is given, set source content id to current
      // content id.
      $item[$object['parentkey']] = $item[$object['key']];
    }
    return translatable_insert($object, $item);
  }
}

/**
 * Insert a new content into translation index.
 */
function translatable_insert($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  translatable_get_object_definition($object);
  
  if ($object['table'] == 'translatable_object') {
    $item[$object['key']] = db_next_id('{translatable_object}_'. $object['key']);
  }
  
  $table = $object['table'];
  unset($object['table']);
  
  $fields = array();
  foreach ($object as $key => $column) {
    $fields['columns'][] = $column;
    if ($key == 'key' || $key == 'parentkey') {
      $fields['types'][] = '%d';
    }
    else {
      $fields['types'][] = "'%s'";
    }
    $fields['values'][] = $item[$column];
  }
  db_query("INSERT INTO {$table} (". implode(',', $fields['columns']) .") VALUES (". implode(',', $fields['types']) .")", $fields['values']);
  return $item[$object['key']];
}

/**
 * Update a content in translation index.
 */
function translatable_update($object, $item) {
  if (!is_array($item)) {
    return FALSE;
  }
  translatable_get_object_definition($object);
  
  $table = $object['table'];
  unset($object['table']);
  
  $fields = array();
  $where  = array();
  foreach ($object as $key => $column) {
    if ($key == 'key') {
      $fields['where'] = $column .' = '. $item[$column];
      continue;
    }
    if ($key == 'parentkey') {
      $fields['assigns'][] = $column .' = %d';
    }
    else {
      $fields['assigns'][] = $column ." = '%s'";
    }
    $fields['values'][] = $item[$column];
  }
  return db_query("UPDATE {$table} SET ". implode(',', $fields['assigns']) ." WHERE ". $fields['where'], $fields['values']);
}

/**
 * Delete a translation reference in translation index.
 *
 * @todo If a source content is deleted, all translations of this node have to
 *       be deleted, too.
 */
function translatable_delete($object, $id) {
  if ($id) {
    translatable_get_object_definition($object);
    db_query('DELETE FROM {%s} WHERE %s = %d', $object['table'], $object['key'], $id);
  }
}

/**
 * Delete all translations for a given condition.
 */
function translatable_delete_all($object, $conditions) {
  if ($conditions) {
    translatable_get_object_definition($object);
    db_query('DELETE FROM {%s} WHERE '. $conditions, $object['table']);
  }
}

